@@ -46,8 +46,6 @@
 
 #define DELAYED_CALLBACK_DELAY (50)
 
-#define ERROR_QUARK (g_quark_from_static_string ("AisleRiot"))
-
 struct _AisleriotGame
 {
   GObject parent_instance;
@@ -58,6 +56,7 @@ struct _AisleriotGame
   guint timeout;
   guint score;
   guint32 seed;
+  guint delayed_call_timeout_id;
 
   time_t start_time;
 
@@ -77,12 +76,14 @@ struct _AisleriotGame
   SCM apply_options_lambda;
   SCM timeout_lambda;
   SCM droppable_lambda;
+  SCM dealable_lambda;
 
   guint click_to_move : 1;
   guint can_undo : 1;
   guint can_redo : 1;
+  guint can_deal : 1;
   guint show_score : 1;
-  guint features : 2; /* enough bits for ALL_FEATURES */
+  guint features : 3; /* enough bits for ALL_FEATURES */
   guint state : 3; /* enough bits for LAST_GAME_STATE */
   guint had_exception : 1;
   guint paused : 1;
@@ -96,6 +97,7 @@ enum
   PROP_0,
   PROP_CAN_UNDO,
   PROP_CAN_REDO,
+  PROP_CAN_DEAL,
   PROP_GAME_FILE,
   PROP_SCORE,
   PROP_STATE,
@@ -118,6 +120,15 @@ G_DEFINE_TYPE (AisleriotGame, aisleriot_game, G_TYPE_OBJECT);
 
 /* helper functions */
 
+static void
+clear_delayed_call (AisleriotGame *game)
+{
+  if (game->delayed_call_timeout_id != 0) {
+    g_source_remove (game->delayed_call_timeout_id);
+    game->delayed_call_timeout_id = 0;
+  }
+}
+
 static void
 set_game_state (AisleriotGame *game,
                 guint state)
@@ -167,7 +178,18 @@ set_game_redoable (AisleriotGame *game,
   }
 }
 
-/* Yes, there is a gconf race condition here, but since this is all
+static void
+set_game_dealable (AisleriotGame *game,
+                   gboolean enabled)
+{
+  if (enabled != game->can_deal) {
+    game->can_deal = enabled != FALSE;
+
+    g_object_notify (G_OBJECT (game), "can-deal");
+  }
+}
+
+/* Yes, there is a race condition here, but since this is all
    per user they're going to have to be either playing impossibly fast
    or using sharing an account. In the later case they won't be caring
    about the statistics. */
@@ -243,18 +265,6 @@ get_slot (AisleriotGame *game,
   return NULL;
 }
 
-#if 0
-static gboolean
-timer_cb ()
-{
-  timeout = 3600;
-  /* All the games return #f and nothing else with this call, but just in
-   * case someone changes their mind in the future. */
-  if (aisleriot_game_timeout_lambda (app_game))
-    aisleriot_game_test_end_of_game (app_game);
-  return 0;
-}
-#endif
 
 /* Scheme helpers */
 
@@ -633,6 +643,18 @@ scm_redo_set_sensitive (SCM in_state)
   return SCM_EOL;
 }
 
+static SCM
+scm_dealable_set_sensitive (SCM in_state)
+{
+  AisleriotGame *game = app_game;
+  gboolean state;
+
+  state = SCM_NFALSEP (in_state) ? TRUE : FALSE;
+  set_game_dealable (game, state);
+
+  return SCM_EOL;
+}
+
 static SCM
 scm_get_feature_word ()
 {
@@ -810,14 +832,29 @@ scm_set_lambda (SCM start_game_lambda,
   game->game_over_lambda = game_over_lambda;
   game->winning_game_lambda = winning_game_lambda;
   game->hint_lambda = hint_lambda;
-  game->get_options_lambda = SCM_CAR (rest);
-  game->apply_options_lambda = SCM_CADR (rest);
-  game->timeout_lambda = SCM_CADDR (rest);
 
-  if (game->features & FEATURE_DROPPABLE)
-    game->droppable_lambda = SCM_CADDDR (rest);
-  else
+  game->get_options_lambda = SCM_CAR (rest);
+  rest = SCM_CDR (rest);
+
+  game->apply_options_lambda = SCM_CAR (rest);
+  rest = SCM_CDR (rest);
+
+  game->timeout_lambda = SCM_CAR (rest);
+  rest = SCM_CDR (rest);
+
+  if (game->features & FEATURE_DROPPABLE) {
+    game->droppable_lambda = SCM_CAR (rest);
+    rest = SCM_CDR (rest);
+  } else {
     game->droppable_lambda = SCM_UNDEFINED;
+  }
+
+  if (game->features & FEATURE_DEALABLE) {
+    game->dealable_lambda = SCM_CAR (rest);
+    rest = SCM_CDR (rest);
+  } else {
+    game->dealable_lambda = SCM_UNDEFINED;
+  }
 
   return SCM_EOL;
 }
@@ -892,7 +929,11 @@ scm_get_timeout ()
 static void
 scm_delayed_call_destroy_data (SCM callback)
 {
+  AisleriotGame *game = app_game;
+
   scm_gc_unprotect_object (callback);
+
+  game->delayed_call_timeout_id = 0;
 }
 
 static gboolean
@@ -901,6 +942,11 @@ scm_execute_delayed_function (SCM callback)
   AisleriotGame *game = app_game;
   CallData data = CALL_DATA_INIT;
 
+  /* We set game->delayed_call_timeout_id to 0 _before_ calling |callback|,
+   * since it might install a new delayed call.
+   */
+  game->delayed_call_timeout_id = 0;
+
   data.lambda = callback;
   data.n_args = 0;
   scm_internal_stack_catch (SCM_BOOL_T,
@@ -912,10 +958,17 @@ scm_execute_delayed_function (SCM callback)
   return FALSE;
 }
 
-/* FIXMEchpe remove the timeout when starting a new game! */
 static SCM
 scm_delayed_call (SCM callback)
 {
+  AisleriotGame *game = app_game;
+
+  /* We can only have one pending delayed call! */
+  if (game->delayed_call_timeout_id != 0) {
+    return scm_throw (scm_from_locale_symbol ("invalid-call"),
+                      scm_cons (scm_from_locale_string ("Already have a delayed callback pending."), SCM_EOL));
+  }
+
   /* We need to protect the callback data from being GC'd until the
    * timeout has run.
    */
@@ -965,6 +1018,25 @@ cscm_init ()
   scm_c_define_gsubr ("_", 1, 0, 0, scm_gettext);
   scm_c_define_gsubr ("undo-set-sensitive", 1, 0, 0, scm_undo_set_sensitive);
   scm_c_define_gsubr ("redo-set-sensitive", 1, 0, 0, scm_redo_set_sensitive);
+  scm_c_define_gsubr ("dealable-set-sensitive", 1, 0, 0, scm_dealable_set_sensitive);
+}
+
+static void
+update_game_dealable (AisleriotGame *game)
+{
+  CallData data = CALL_DATA_INIT;
+
+  if ((game->features & FEATURE_DEALABLE) == 0)
+    return;
+
+  data.lambda = game->dealable_lambda;
+  data.n_args = 0;
+  scm_internal_stack_catch (SCM_BOOL_T,
+                            cscmi_call_lambda, &data,
+                            cscmi_catch_handler, NULL);
+
+  g_print ("dealable returned %d\n", SCM_NFALSEP (data.retval));
+  set_game_dealable (game, SCM_NFALSEP (data.retval));
 }
 
 static void
@@ -1025,7 +1097,7 @@ cscmi_eval_installed_file (const char *filename,
   /* FIXMEchpe: install exception handlers and set error if an exception occurs while loading the file? */
 
   if (!g_str_has_suffix (filename, ".scm")) {
-    g_set_error (error, ERROR_QUARK, GAME_ERROR_GENERIC,
+    g_set_error (error, AISLERIOT_GAME_ERROR, GAME_ERROR_GENERIC,
                  "%s is not a scheme file", filename);
     return FALSE;
   }
@@ -1038,7 +1110,7 @@ cscmi_eval_installed_file (const char *filename,
     return TRUE;
   }
 
-  g_set_error (error, ERROR_QUARK, GAME_ERROR_GENERIC,
+  g_set_error (error, AISLERIOT_GAME_ERROR, GAME_ERROR_GENERIC,
                _("Aisleriot cannot load the file \"%s\". "
                  "Please check your Aisleriot installation."),
                path);
@@ -1087,6 +1159,7 @@ aisleriot_game_finalize (GObject *object)
     update_statistics (game);
   }
 
+  clear_delayed_call (game);
   clear_slots (game, FALSE);
   g_ptr_array_free (game->slots, TRUE);
 
@@ -1112,6 +1185,9 @@ aisleriot_game_get_property (GObject *object,
     case PROP_CAN_REDO:
       g_value_set_boolean (value, game->can_redo);
       break;
+    case PROP_CAN_DEAL:
+      g_value_set_boolean (value, game->can_deal);
+      break;
     case PROP_GAME_FILE:
       g_value_set_string (value, game->game_file);
       break;
@@ -1206,6 +1282,13 @@ aisleriot_game_class_init (AisleriotGameClass *klass)
                            FALSE,
                            G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
+  g_object_class_install_property
+    (gobject_class,
+     PROP_CAN_DEAL,
+     g_param_spec_boolean ("can-deal", NULL, NULL,
+                           FALSE,
+                           G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+
   g_object_class_install_property
     (gobject_class,
      PROP_GAME_FILE,
@@ -1233,6 +1316,23 @@ aisleriot_game_class_init (AisleriotGameClass *klass)
 
 /* public API */
 
+/**
+ * aisleriot_game_error_quark:
+ *
+ * Returns: the #GQuark used for errors
+ */
+GQuark
+aisleriot_game_error_quark (void)
+{
+  static GQuark quark = 0;
+
+  if (G_UNLIKELY (quark == 0)) {
+    quark = g_quark_from_static_string ("AisleRiotGameError");
+  }
+
+  return quark;
+}
+
 /**
  * aisleriot_game_new:
  *
@@ -1358,6 +1458,8 @@ aisleriot_game_undo_move (AisleriotGame *game)
   }
 
   cscmi_eval_string ("(undo)");
+
+  update_game_dealable (game);
 }
 
 /**
@@ -1405,7 +1507,7 @@ aisleriot_game_load_game (AisleriotGame *game,
    * as a key, we also require it to be valid UTF-8 !
    */
   if (!g_utf8_validate (game_file, -1, NULL)) {
-    g_set_error (error, ERROR_QUARK, GAME_ERROR_GENERIC,
+    g_set_error (error, AISLERIOT_GAME_ERROR, GAME_ERROR_GENERIC,
                  "Invalid UTF-8");
     return FALSE;
   }
@@ -1419,10 +1521,12 @@ aisleriot_game_load_game (AisleriotGame *game,
 
   g_object_freeze_notify (object);
 
+  clear_delayed_call (game);
   set_game_state (game, GAME_UNINITIALISED);
   set_game_score (game, 0);
   set_game_undoable (game, FALSE);
   set_game_redoable (game, FALSE);
+  set_game_dealable (game, FALSE);
   game->features = 0;
   game->had_exception = FALSE;
 
@@ -1439,7 +1543,7 @@ aisleriot_game_load_game (AisleriotGame *game,
     } else {
       g_error_free (err);
 
-      g_set_error (error, ERROR_QUARK, GAME_ERROR_FALLBACK,
+      g_set_error (error, AISLERIOT_GAME_ERROR, GAME_ERROR_FALLBACK,
                    "%s %s",
                    _("Aisleriot cannot find the last game you played."),
                    _("This usually occurs when you run an older version of Aisleriot "
@@ -1503,7 +1607,8 @@ aisleriot_game_new_game (AisleriotGame *game,
     update_statistics (game);
   }
 
-  /* The game isn't actually in progress until the user makes a move. */
+  clear_delayed_call (game);
+  /* The game isn't actually in progress until the user makes a move */
   set_game_state (game, GAME_BEGIN);
   set_game_score (game, 0);
   set_game_undoable (game, FALSE);
@@ -1528,6 +1633,8 @@ aisleriot_game_new_game (AisleriotGame *game,
     scm_c_eval_string ("(start-game)");
   } while (!cscmi_game_over_lambda ());
 
+  update_game_dealable (game);
+
   g_object_thaw_notify (object);
 
   g_signal_emit (game, signals[GAME_NEW], 0);
@@ -1896,7 +2003,7 @@ aisleriot_game_discard_move (AisleriotGame *game)
 }
 
 /**
- * aisleriot_game_test_end_of_game:
+ * aisleriot_game_update_game_state:
  * @game:
  *
  * Tests whether the game is over.
@@ -1906,6 +2013,8 @@ aisleriot_game_test_end_of_game (AisleriotGame *game)
 {
   aisleriot_game_end_move (game);
 
+  update_game_dealable (game);
+
   if (game->state < GAME_OVER) {
     if (!cscmi_game_over_lambda ()) {
       guint new_state;
@@ -1949,3 +2058,22 @@ aisleriot_game_generate_exception (AisleriotGame *game)
 {
   cscmi_eval_string ("(how-about-a-nice-crash?)");
 }
+
+/**
+ * aisleriot_game_deal_cards:
+ * @game:
+ *
+ * Deals the next card or cards, if possible.
+ *
+ * Returns: %TRUE iff cards were dealt
+ */
+void
+aisleriot_game_deal_cards (AisleriotGame *game)
+{
+  aisleriot_game_record_move (game, -1, NULL, 0);
+
+  cscmi_eval_string ("(do-deal-next-cards)");
+    
+  aisleriot_game_end_move (game);
+  aisleriot_game_test_end_of_game (game);
+}
