@@ -70,16 +70,18 @@ typedef struct {
 
 /* The game */
 
-#define AISLERIOT_TYPE_GAME		(aisleriot_game_get_type ())
-#define AISLERIOT_GAME(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), AISLERIOT_TYPE_GAME, AisleriotGame))
-#define AISLERIOT_GAME_CLASS(k)	(G_TYPE_CHECK_CLASS_CAST((k), AISLERIOT_TYPE_GAME, AisleriotGameClass))
-#define AISLERIOT_IS_GAME(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), AISLERIOT_TYPE_GAME))
-#define AISLERIOT_IS_GAME_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), AISLERIOT_TYPE_GAME))
-#define AISLERIOT_GAME_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), AISLERIOT_TYPE_GAME, AisleriotGameClass))
+#define AISLERIOT_TYPE_GAME         (aisleriot_game_get_type ())
+#define AISLERIOT_GAME(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), AISLERIOT_TYPE_GAME, AisleriotGame))
+#define AISLERIOT_GAME_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST((k), AISLERIOT_TYPE_GAME, AisleriotGameClass))
+#define AISLERIOT_IS_GAME(o)        (G_TYPE_CHECK_INSTANCE_TYPE ((o), AISLERIOT_TYPE_GAME))
+#define AISLERIOT_IS_GAME_CLASS(k)  (G_TYPE_CHECK_CLASS_TYPE ((k), AISLERIOT_TYPE_GAME))
+#define AISLERIOT_GAME_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), AISLERIOT_TYPE_GAME, AisleriotGameClass))
 
 #define DEFAULT_VARIATION  "klondike.scm"
 #define FREECELL_VARIATION "freecell.scm"
 
+#define AISLERIOT_GAME_ERROR  (aisleriot_game_error_quark ())
+
 typedef struct _AisleriotGame AisleriotGame;
 typedef GObjectClass AisleriotGameClass;
 
@@ -89,9 +91,10 @@ enum {
 };
 
 typedef enum {
-  FEATURE_DROPPABLE = 1 << 0,
-  FEATURE_SCORE_HIDDEN = 1 << 1,
-  ALL_FEATURES = 0x3
+  FEATURE_DROPPABLE     = 1 << 0,
+  FEATURE_SCORE_HIDDEN  = 1 << 1,
+  FEATURE_DEALABLE      = 1 << 2,
+  ALL_FEATURES          = 0x7
 } AisleriotGameFeatures;
 
 typedef enum {
@@ -104,6 +107,8 @@ typedef enum {
   LAST_GAME_STATE
 } AisleriotGameState;
 
+GQuark aisleriot_game_error_quark (void);
+
 GType aisleriot_game_get_type (void);
 
 AisleriotGame *aisleriot_game_new (void);
@@ -184,6 +189,8 @@ void aisleriot_game_test_end_of_game (AisleriotGame * game);
 
 void aisleriot_game_generate_exception (AisleriotGame * game);
 
+void aisleriot_game_deal_cards (AisleriotGame *game);
+
 G_END_DECLS
 
 #endif /* !AISLERIOT_GAME_H */
