@@ -1,6 +1,8 @@
 // -*- mode:C++; tab-width:8; c-basic-offset:8; indent-tabs-mode:nil -*-
-/* Blackjack - events.cpp
- * Copyright (C) 2003 William Jon McCann <mccann@jhu.edu>
+/*
+ * Blackjack - events.cpp
+ *
+ * Copyright (C) 2003-2005 William Jon McCann <mccann@jhu.edu>
  *
  * This game is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,18 +22,11 @@
 
 #include <config.h>
 
-#ifdef HAVE_ALLOCA_H
-#include <alloca.h>
-#endif
-
-#include <sys/types.h>
-#include <string.h>
-#include <stdlib.h>
-#include <time.h>
-#include <dirent.h>
-#include <ctype.h>
-#include <iostream>
 #include <math.h>
+#include <glib/gi18n.h>
+#include <gdk/gdkkeysyms.h>
+#include <gtk/gtk.h>
+
 using namespace std;
 
 #include "events.h"
@@ -42,6 +37,7 @@ using namespace std;
 #include "card.h"
 #include "menu.h"
 #include "chips.h"
+#include "press_data.h"
 
 #include "hand.h"
 #include "game.h"
@@ -135,7 +131,8 @@ set_cursor_by_location (int cursor, int x, int y)
 gboolean
 bj_event_key_press (GtkWidget *widget, GdkEventKey *event, void *d)
 {
-        guint key;
+        guint    key;
+        gboolean events_pending = bj_hand_events_pending ();
 
         // ignore clicks during actions
         if (events_pending)
@@ -145,29 +142,15 @@ bj_event_key_press (GtkWidget *widget, GdkEventKey *event, void *d)
 
         // Carry out player option.
         switch (key) {
-        case KEY_S :
-                bj_hand_stand ();
-                break;
-        case KEY_H :
-                bj_hand_hit_with_delay ();
-                break;
-        case KEY_D : 
-                bj_hand_double ();
-                break;
-        case KEY_P :
-                bj_hand_split ();
-                break;
-        case KEY_R :
-                bj_hand_surrender ();
-                break;
         case KEY_ENTER :
                 bj_hand_new_deal ();
+                return TRUE;
                 break;
         default:
                 break;
         }
 
-        return true;
+        return FALSE;
 }
 
 int
@@ -184,10 +167,6 @@ waiting_for_mouse_up (void)
 
 /* Button press statuses */
 
-void end_of_game_test ()
-{
-}
-
 static void
 drop_moving_cards (gint x, gint y)
 {
@@ -220,16 +199,9 @@ drop_moving_cards (gint x, gint y)
                          hslot->pixelx + hslot->width - width, 
                          hslot->pixely + hslot->height - height);
 
-        bj_draw_refresh_screen ();
+        bj_press_data_free ();
 
-        gdk_window_hide (press_data->moving_cards);
-  
-        if (press_data->moving_pixmap)
-                g_object_unref (press_data->moving_pixmap);
-        if (press_data->moving_mask)
-                g_object_unref (press_data->moving_mask);
-        press_data->moving_pixmap = NULL;
-        press_data->moving_mask = NULL;
+        bj_draw_refresh_screen ();
 
         if (do_split)
                 bj_hand_split ();
@@ -279,18 +251,8 @@ drop_moving_chips (gint x, gint y)
                 bj_chip_stack_update_length (chip_stack_press_data->hstack);
         }
 
-        chip_stack_press_data->chips = NULL;
-
+        bj_chip_stack_press_data_free ();
         bj_draw_refresh_screen ();
-
-        gdk_window_hide (chip_stack_press_data->moving_chips);
-  
-        if (chip_stack_press_data->moving_pixmap)
-                g_object_unref (chip_stack_press_data->moving_pixmap);
-        if (chip_stack_press_data->moving_mask)
-                g_object_unref (chip_stack_press_data->moving_mask);
-        chip_stack_press_data->moving_pixmap = NULL;
-        chip_stack_press_data->moving_mask = NULL;
 }
 
 static gint
@@ -341,7 +303,6 @@ handle_slot_pressed (GdkEventButton *event, hslot_type hslot, gint cardid)
         else if (double_click) {
                 press_data->status = STATUS_NONE;
                 //bj_draw_refresh_screen ();
-                //end_of_game_test ();
                 return TRUE;
         }
         return TRUE;
@@ -422,12 +383,12 @@ handle_chip_stack_pressed (GdkEventButton *event,
 gint
 bj_event_button_press (GtkWidget *widget, GdkEventButton *event, void *d)
 {
+        gboolean events_pending = bj_hand_events_pending ();
 
         // ignore clicks during actions
         if (events_pending)
                 return TRUE;
 
-
         if (event->button != 1)
                 return TRUE;
 
@@ -476,16 +437,19 @@ bj_slot_clicked ()
 gint
 bj_event_button_release (GtkWidget *widget, GdkEventButton *event, void *d)
 {
+        gboolean events_pending = bj_hand_events_pending ();
+
+        // ignore clicks during actions
+        if (events_pending)
+                return TRUE;
+
         switch (press_data->status) {
         case STATUS_IS_DRAG:
                 press_data->status = STATUS_NONE;
                 drop_moving_cards ((gint)event->x, (gint)event->y);
                 break;
         case STATUS_SHOW:
-                press_data->status = STATUS_NONE;
-                gdk_window_hide (press_data->moving_cards);
-                press_data->moving_pixmap = NULL;
-                press_data->moving_mask = NULL;
+                bj_press_data_free ();
                 break;
     
         case STATUS_MAYBE_DRAG:
@@ -505,10 +469,7 @@ bj_event_button_release (GtkWidget *widget, GdkEventButton *event, void *d)
                 drop_moving_chips ((gint)event->x, (gint)event->y);
                 break;
         case STATUS_SHOW:
-                chip_stack_press_data->status = STATUS_NONE;
-                gdk_window_hide (chip_stack_press_data->moving_chips);
-                chip_stack_press_data->moving_pixmap = NULL;
-                chip_stack_press_data->moving_mask = NULL;
+                bj_chip_stack_press_data_free ();
                 break;
     
         case STATUS_MAYBE_DRAG:
@@ -720,6 +681,7 @@ bj_event_playing_area_configure (GtkWidget *widget, GdkEventConfigure *event)
                 if (window_width == event->width && window_height == event->height)
                         return TRUE;
                 g_object_unref (surface);
+                surface = NULL;
         }
   
         if (!draw_gc) {
@@ -756,19 +718,3 @@ bj_event_playing_area_configure (GtkWidget *widget, GdkEventConfigure *event)
 
         return FALSE;
 }
-
-gint
-bj_event_configure (GtkWidget *widget, GdkEventConfigure *event)
-{
-        GConfClient *gconf_client = gconf_client_get_default ();
-
-        gconf_client_set_int (gconf_client, GCONF_KEY_WIDTH,
-                              event->width, NULL);
-        gconf_client_set_int (gconf_client, GCONF_KEY_HEIGHT,
-                              event->height, NULL);
-
-        g_object_unref (gconf_client);
-
-        return FALSE;
-}
-
