@@ -23,6 +23,8 @@
 #include <gnome.h>
 #include <string.h>
 #include <games-stock.h>
+#include <games-scores.h>
+#include <games-scores-dialog.h>
 
 #include "gbdefs.h"
 #include "menu.h"
@@ -163,7 +165,7 @@ show_toolbar_cb (GtkAction * action, gpointer data)
   gboolean state;
 
   state = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
-  gconf_set_show_toolbar (state);
+  conf_set_show_toolbar (state);
 }
 
 
@@ -197,7 +199,7 @@ properties_cb (GtkAction * action, gpointer data)
 static void
 scores_cb (GtkAction * action, gpointer data)
 {
-  show_scores (0);
+  show_scores (0, FALSE);
 }
 
 
@@ -260,10 +262,11 @@ about_cb (GtkAction * action, gpointer data)
 			 "version", VERSION,
 			 "copyright", "Copyright \xc2\xa9 1998-2007 Mark Rae",
 			 "license", license,
-			 "comments", _("Based on classic BSD Robots."),
+		         "website-label", _("GNOME Games web site"),
+			 "comments", _("Based on classic BSD Robots.\n\nRobots is a part of GNOME Games."),
 			 "authors", authors,
 			 "documenters", documenters,
-			 "translator_credits", _("translator-credits"),
+			 "translator-credits", _("translator-credits"),
 			 "logo-icon-name", "gnome-robots",
 			 "website",
 			 "http://www.gnome.org/projects/gnome-games/",
@@ -310,12 +313,14 @@ fullscreen_cb (GtkAction * action)
 }
 
 /* Just in case something else takes us to/from fullscreen. */
-void
+gboolean
 window_state_cb (GtkWidget * widget, GdkEventWindowState * event)
 {
   if (event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN)
     set_fullscreen_actions (event->new_window_state
 			    & GDK_WINDOW_STATE_FULLSCREEN);
+    
+  return FALSE;
 }
 
 /**
@@ -372,34 +377,10 @@ create_game_menus (GtkUIManager * ui_manager)
 void
 update_score_state (void)
 {
-  gchar **names = NULL;
-  gfloat *scores = NULL;
-  time_t *scoretimes = NULL;
-  gint top;
-  gchar *sbuf = NULL;
+  GList *top;
 
-  if (properties_super_safe_moves ()) {
-    sbuf = g_strdup_printf ("%s-super-safe",
-			    game_config_filename (current_game_config ()));
-  } else if (properties_safe_moves ()) {
-    sbuf = g_strdup_printf ("%s-safe",
-			    game_config_filename (current_game_config ()));
-  } else {
-    sbuf = g_strdup_printf ("%s",
-			    game_config_filename (current_game_config ()));
-  }
-
-  top = gnome_score_get_notable (GAME_NAME, sbuf,
-				 &names, &scores, &scoretimes);
-  g_free (sbuf);
-  if (top > 0) {
-    gtk_action_set_sensitive (scores_action, TRUE);
-    g_strfreev (names);
-    g_free (scores);
-    g_free (scoretimes);
-  } else {
-    gtk_action_set_sensitive (scores_action, FALSE);
-  }
+  top = games_scores_get (highscores);
+  gtk_action_set_sensitive (scores_action, top != NULL);
 }
 
 void
@@ -430,4 +411,6 @@ connect_toolbar_toggle (GtkWidget * toolbar)
 		    G_CALLBACK (toggle_toolbar_cb), toolbar);
   gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (toolbar_toggle_action),
 				properties_show_toolbar ());
+
+  toggle_toolbar_cb (GTK_TOGGLE_ACTION (toolbar_toggle_action), toolbar);
 }
