@@ -795,7 +795,8 @@ class UI (gconf_wrapper.GConfWrapper):
 	about.set_wrap_license(True)
         about.set_comments(DESCRIPTION)
         about.set_authors(AUTHORS)
-        about.set_website("http://www.gnome.org/projects/gnome-games/")
+        about.set_website(WEBSITE)
+        about.set_website_label(WEBSITE_LABEL)
         about.set_logo_icon_name("gnome-sudoku")
         about.set_translator_credits(_("translator-credits"))
         about.connect("response", lambda d, r: d.destroy())
