@@ -20,8 +20,10 @@
 #ifndef GAMES_FILES_H
 #define GAMES_FILES_H
 
-G_BEGIN_DECLS
 #include <gtk/gtk.h>
+
+G_BEGIN_DECLS
+
 typedef struct _GamesFileList GamesFileList;
 typedef struct _GamesFileListClass GamesFileListClass;
 
@@ -36,6 +38,7 @@ struct _GamesFileListClass {
 };
 
 #define GAMES_FILE_LIST_TYPE (games_file_list_get_type ())
+#define GAMES_FILE_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GAMES_FILE_LIST_TYPE, GamesFileList))
 
 GType games_file_list_get_type (void);
 
@@ -61,8 +64,14 @@ games_file_list_new (const gchar * glob, ...)
      };
 
      GtkWidget *games_file_list_create_widget (GamesFileList * filelist,
-					       gchar * selection,
+					       const gchar * selection,
 					       guint flags);
 
+GamesFileList *games_file_list_card_themes (gboolean scalable);
+
+gchar *games_build_filename (const gchar *path, const gchar *filename);
+gchar *games_path_runtime_fix (gchar *path);
+
 G_END_DECLS
+
 #endif /* GAMES_FILES_H */
